<?php
/**
 * @version   script.php 2016-08-10 08:20:00 UTC gibiwatch
 * @package   NoNotification system plugin
 * @author    Watchful
 * @authorUrl https://watchful.li
 * @copyright Copyright (C) 2016, Watchful
 * @license   GNU/GPL v3 or later
 */
defined('_JEXEC') or die;

if (!class_exists('plgSystemNonotificationInstallerScript'))
{
	class plgSystemNonotificationInstallerScript
	{
		static $hasRan;

		/**
		 * @var \JDatabaseDriver
		 * @since v1.0
		 */
		private $db;

		/**
		 * @param $type
		 * @param $parent
		 *
		 * @since v1.0
		 * @return bool
		 */
		public function postflight($type, $parent)
		{
			// Since we call the uninstaller while we are still installing, we don't want to create a loop  
			if (static::$hasRan) {
				return;
			}
			static::$hasRan = true;

			\JLog::addLogger(array("text_file" => "nonotification.log.php"), \JLog::ALL, 'nonotification');
			\JLog::add("On preflight", \JLog::DEBUG, 'nonotification');
			$this->db = \JFactory::getDbo();
			$this->disableNotificationPlugin();
			$this->uninstallItself();

			return true;
		}

		/**
		 * @since v1.0
		 * @return bool
		 */
		private function disableNotificationPlugin()
		{
			if (!$this->isCompatibleJoomlaVersion())
			{
				return true;
			}

			$this->db = JFactory::getDbo();
			$this->db->setQuery($this->getDisableQuery());

			try
			{
				$this->db->execute();
			}
			catch (\Exception $ex)
			{
				\JLog::add("Exception: " . $ex->getMessage(), \JLog::ERROR, 'nonotification');

				return false;
			}

			return true;
		}

		/**
		 * @since v1.0
		 * @return bool
		 */
		private function uninstallItself()
		{
			$itself = $this->findNonotificationExtension();
			\JInstaller::getInstance()->uninstall($itself->type, $itself->extension_id);

			return true;
		}

		/**
		 * @since v1.0
		 * @return bool
		 */
		private function isCompatibleJoomlaVersion()
		{
			$version = new JVersion();

			return $version->isCompatible("3.6.0");
		}

		/**
		 * @since v1.0
		 * @return \JDatabaseQuery
		 */
		private function getDisableQuery()
		{
			return $this->db->getQuery(true)
				->update("#__extensions")
				->set("enabled = 0")
				->where("type = 'plugin'")
				->where("element = 'updatenotification'")
				->where("folder = 'system'");
		}

		/**
		 * @since v1.0
		 * @return \stdClass
		 */
		private function findNonotificationExtension()
		{
			$query = $this->db->getQuery(true)
				->select('extension_id')
				->select('type')
				->from('#__extensions')
				->where("`element` = 'nonotification'")
				->where("`folder` = 'system'")
				->where("`type` = 'plugin'");

			$this->db->setQuery($query);
			try
			{
				return $this->db->loadObject();
			}
			catch (\Exception $ex)
			{
				\JLog::add("[findNonotificationExtension] Exception: " . $ex->getMessage(), \JLog::ERROR, 'nonotification');
			}

		}
	}
}